/******************************************************************************
 * Spine Runtimes Software License v2.5
 *
 * Copyright (c) 2013-2016, Esoteric Software
 * All rights reserved.
 *
 * You are granted a perpetual, non-exclusive, non-sublicensable, and
 * non-transferable license to use, install, execute, and perform the Spine
 * Runtimes software and derivative works solely for personal or internal
 * use. Without the written permission of Esoteric Software (see Section 2 of
 * the Spine Software License Agreement), you may not (a) modify, translate,
 * adapt, or develop new applications using the Spine Runtimes or otherwise
 * create derivative works or improvements of the Spine Runtimes or (b) remove,
 * delete, alter, or obscure any trademarks or any copyright, trademark, patent,
 * or other intellectual property or proprietary rights notices on or in the
 * Software, including any copy thereof. Redistributions in binary or source
 * form must include this license and terms.
 *
 * THIS SOFTWARE IS PROVIDED BY ESOTERIC SOFTWARE "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL ESOTERIC SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES, BUSINESS INTERRUPTION, OR LOSS OF
 * USE, DATA, OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#ifndef Spine_VertexEffect_h
#define Spine_VertexEffect_h

#include <spine/SpineObject.h>
#include <spine/MathUtil.h>

namespace spine {

class Skeleton;
class Color;

class SP_API VertexEffect: public SpineObject {
public:
	virtual void begin(Skeleton& skeleton) = 0;
	virtual void transform(float& x, float& y, float &u, float &v, Color &light, Color &dark) = 0;
	virtual void end() = 0;
};

class SP_API JitterVertexEffect: public VertexEffect {
public:
	JitterVertexEffect(float jitterX, float jitterY);

	void begin(Skeleton& skeleton);
	void transform(float& x, float& y, float &u, float &v, Color &light, Color &dark);
	void end();

	void setJitterX(float jitterX);
	float getJitterX();

	void setJitterY(float jitterY);
	float getJitterY();

protected:
	float _jitterX;
	float _jitterY;
};

class SP_API SwirlVertexEffect: public VertexEffect {
public:
	SwirlVertexEffect(float radius, Interpolation &interpolation);

	void begin(Skeleton& skeleton);
	void transform(float& x, float& y, float &u, float &v, Color &light, Color &dark);
	void end();

	void setCenterX(float centerX);
	float getCenterX();

	void setCenterY(float centerY);
	float getCenterY();

	void setRadius(float radius);
	float getRadius();

	void setAngle(float angle);
	float getAngle();

	void setWorldX(float worldX);
	float getWorldX();

	void setWorldY(float worldY);
	float getWorldY();

protected:
	float _centerX;
	float _centerY;
	float _radius;
	float _angle;
	float _worldX;
	float _worldY;

	Interpolation& _interpolation;
};
}

#endif /* Spine_VertexEffect_h */
